import re
from unicodedata import normalize
from email_validator import validate_email, EmailNotValidError


class Cleansing(object):
    class Normalize:
        def __init__(self, string):
            self.string = string

        def execute(self):
            try:
                rem = normalize('NFKD', self.string) \
                    .encode('ASCII', 'ignore') \
                    .decode('ASCII')
                return rem

            except ValueError as e:
                pass

    class Prepositions:
        def __init__(self, string, prepositions):
            self.string = string
            self.prepositions = prepositions

        def execute(self):
            try:
                ajust_name = (' '.join(word if word in self.prepositions else word.title()
                                       for word in self.string.capitalize().split(' ')))
                return ajust_name

            except ValueError as e:
                pass

    class Phone:
        def __init__(self, string):
            self.string = string

        def phone_cleansing(self):
            number_pattern = re.compile('\\d')

            try:
                number = re.sub(number_pattern, '', self.string)
                x = len(self.string)

                if x < 8:
                    self.string = None

                elif x > 11:
                    self.string = number[x - 11:]

                return self.string

            except Exception as e:
                raise Exception(str(e))

        def phone_validate(self):
            fixed_pattern = re.compile('(\\d{3})?(\\d{2})?(\\d{4})(\\d{4})')
            mobile_pattern = re.compile('(\\d{3})?(\\d{2})?(\\d{5})(\\d{4})')

            try:
                phone_number = self.phone_cleansing()

                x = len(str(phone_number))

                if x == 8 or x == 10:
                    re.match(fixed_pattern, phone_number).group(0)

                    return 'Phone'

                elif x == 9 or x == 11:
                    re.match(mobile_pattern, phone_number).group(0)

                    return 'Mobile'

                else:
                    return 'Invalid'

            except Exception as e:
                raise Exception(str(e))

    class EMail:
        def __init__(self, string):
            self.string = string

        def email_validate(self):
            if self.string is not None:
                try:
                    validate_email(self.string)

                    return True

                except EmailNotValidError as e:
                    print(str(e))

                    return False



import base64
from Crypto import Random
from Crypto.Cipher import AES
from simple_salesforce import Salesforce


class SalesForce(object):
    class Auth:
        def __init__(self, sf_username, sf_pwd, sf_security_token):
            self.sf_username = sf_username
            self.sf_pwd = sf_pwd
            self.sf_security_token = sf_security_token

        def salesforce_login(self):
            try:
                sf = Salesforce(
                    username=self.sf_username,
                    password=self.sf_pwd,
                    security_token=self.sf_security_token)

            except ValueError as e:
                print(e)

            return sf


class SentryBay(object):
    class EncodeData:
        def __init__(self, sb_key, sb_data):
            self.sb_key = sb_key
            self.sb_data = sb_data

        def encrypt_data(self):
            data = self.sb_data + self.padding()

            iv_created = Random.new().read(AES.block_size)

            cipher = AES.new(self.sb_key.encode('utf-8'), AES.MODE_CBC, iv_created)

            encrypted = cipher.encrypt(str.encode(data))

            result = base64.b64encode(iv_created).decode('utf-8'), base64.b64encode(encrypted).decode('utf-8')

            return result

        def padding(self):
            try:
                size = len(self.sb_data)

                i = int(size / AES.block_size)

                j = AES.block_size * i

                if j < size:
                    j += AES.block_size

                return "".ljust(j - size, '\x00')

            except Exception as e:
                raise Exception(str(e))



class DeltaLake(object):
    class LandingZone:
        def __init__(self, source_dataframe, file_path, stream_query):
            self.source_dataframe = source_dataframe
            self.file_path = file_path
            self.stream_query = stream_query

        def write_stream(self):
            try:
                (self.source_dataframe \
                 .writeStream.format('delta') \
                 .option('checkpointLocation', self.file_path + '/_checkpoint') \
                 .queryName(self.stream_query) \
                 .outputMode('append') \
                 .start(self.file_path) \
                 .awaitTermination())

                return

            except ValueError as e:
                return print(e)





import pymongo
import psycopg2
import cx_Oracle
import urllib.parse


class Databases(object):
    class Oracle:
        def __init__(self, oracle_user, oracle_pwd, oracle_dsn):
            self.oracle_user = oracle_user
            self.oracle_pwd = oracle_pwd
            self.oracle_dsn = oracle_dsn

        def conn_oracle(self):
            try:
                cnxn = cx_Oracle.connect(user=self.oracle_user,
                                         password=self.oracle_pwd,
                                         dsn=self.oracle_dsn)

                return cnxn

            except ValueError as e:
                return print(e)

    class Redshift:
        def __init__(self, rs_db, rs_host, rs_port,
                     rs_user, rs_pwd):
            self.rs_db = rs_db
            self.rs_host = rs_host
            self.rs_port = int(rs_port)
            self.rs_user = rs_user
            self.rs_pwd = rs_pwd

        def conn_redshift(self):
            try:
                cnxn = psycopg2.connect(dbname=self.rs_db,
                                        host=self.rs_host,
                                        port=self.rs_port,
                                        user=self.rs_user,
                                        password=self.rs_pwd)

                return cnxn

            except ValueError as e:
                return print(e)

    class DocumentDB:
        def __init__(self, ddb_host, ddb_port, ddb_user,
                     ddb_pwd, ddb_public_key):
            self.ddb_host = ddb_host
            self.ddb_port = int(ddb_port)
            self.ddb_user = urllib.parse.quote_plus(ddb_user)
            self.ddb_pwd = urllib.parse.quote_plus(ddb_pwd)

            self.ddb_public_key = ddb_public_key

        def conn_ddb(self):
            try:
                engine = pymongo.MongoClient

                cnxn = engine('mongodb://%s:%s@%s:%s'
                              % (self.ddb_user, self.ddb_pwd,
                                 self.ddb_host, self.ddb_port),
                              tls=True,
                              tlsCAFile=self.ddb_public_key
                              )
                return cnxn

            except ValueError as e:
                return print(e)


class SparkDatabases(object):
    class Oracle:
        def __init__(self, spark_instance, host, port, service, user, pwd, arg):
            self.spark_instance = spark_instance
            self.host = host
            self.port = port
            self.service = service
            self.user = user
            self.pwd = pwd
            self.arg = arg

        def read_oracle(self):
            try:
                data = (self.spark_instance.read \
                        .format("jdbc") \
                        .option("url",
                                "jdbc:oracle:thin:@(DESCRIPTION="
                                "(ADDRESS=(PROTOCOL=TCP)"
                                "(HOST=" + self.host + ")" \
                                                       "(PORT=" + self.port + "))" \
                                                                              "(CONNECT_DATA=(SERVER=DEDICATED)" \
                                                                              "(SERVICE_NAME=" + self.service + ")))") \
                        .option("driver", "oracle.jdbc.driver.OracleDriver") \
                        .option("user", self.user) \
                        .option("password", self.pwd) \
                        .option("numPartitions", 4) \
                        .option("dbtable", self.arg)) \
                    .load()

                return data

            except ValueError as e:
                return print(e)

        def write_oracle(self):
            try:
                data = (self.spark_instance.write \
                        .format("jdbc") \
                        .option("url",
                                "jdbc:oracle:thin:@(DESCRIPTION="
                                "(ADDRESS=(PROTOCOL=TCP)"
                                "(HOST=" + self.host + ")" \
                                                       "(PORT=" + self.port + "))" \
                                                                              "(CONNECT_DATA=(SERVER=DEDICATED)" \
                                                                              "(SERVICE_NAME=" + self.service + ")))") \
                        .option("driver", "oracle.jdbc.driver.OracleDriver") \
                        .option("user", self.user) \
                        .option("password", self.pwd) \
                        .option("mode", "append") \
                        .option("dbtable", self.arg)) \
                    .save()

                return data

            except ValueError as e:
                return print(e)

    class Informix:
        def __init__(self, spark_instance, host, port, service, status, user, pwd, arg):
            self.spark_instance = spark_instance
            self.host = host
            self.port = port
            self.service = service
            self.status = status
            self.user = user
            self.pwd = pwd
            self.arg = arg

        def conn_informix(self):
            try:
                data = (self.spark_instance.read \
                        .format("jdbc") \
                        .option("url",
                                "jdbc:informix-sqli://"
                                + self.host + ":" \
                                + self.port + "/" \
                                + self.service + "=" \
                                + self.status) \
                        .option("driver", "com.informix.jdbc.IfxDriver") \
                        .option("user", self.user) \
                        .option("password", self.pwd) \
                        .option("numPartitions", 4) \
                        .option("dbtable", self.arg)) \
                    .load()

                return data

            except ValueError as e:
                return print(e)
